from threading import Lock


class Singleton(type):
    """
    This is a thread-safe implementation of Singleton.
    @see https://refactoring.guru/pt-br/design-patterns/singleton/python/example#example-1
    """
    _instances = {}
    _lock: Lock = Lock()

    def __call__(cls, *args, **kwargs):
        """
        Possible changes to the value of the `__init__` argument do not affect
        the returned instance.
        """
        with cls._lock:
            if cls not in cls._instances:
                instance = super().__call__(*args, **kwargs)
                cls._instances[cls] = instance
        return cls._instances[cls]
